/*
 *  - l1[] is a compact memory array for base configurations
 *  - configs are stored in a bytewise form
 *  - access it via getl1() and putl1()
 */

/* #include "spinsdef.h" */	/* define tbase,tbase1 (define later here!) */
#ifndef _TBASE_H
#define _TBASE_H 1

#include <stdio.h>
#include "config.h"	/* define tbase,tbase1 (define later here!) */
#include "spins.h"	/* define tbase,tbase1 (later in hilbert.h?) */

extern unsigned char *l1;	/* packed sym-configs of local node */

#define L1_PACKED 0   /* 2016-01: old=1 packed compact + slow, new=0 fast */

#if defined(SSE2_tbase) && L1_PACKED
# error "L1_PACKED must be disabled for SSE2_tbase" 
#endif
#if (NN&(NN-1))==0 && L1_PACKED  /* log2-tbase */
# error "L1_PACKED makes no sence for log2 NN, SH slowdown only" 
#endif

#if   L1_PACKED
#define Bsize  ((CM(1,2,2)*NN-1)/8+1)    /* minimum char[] space (Bytes) */
#else
#define Bsize  (S_tbase)    /* log2 space in Bytes = faster l1 access */
#endif

extern char FILE_l1[32];        /* tmp/l1_%3d$(mpi_i).dat -> unused? */
const extern char *FILE_l10;    /* tmp/l1_0000.dat */
extern FILE *fl1; 		/* only for mmap, hide later! */

int save_l1(unsigned char *l1, ULong n1, ULong ofs);
int load_l1(unsigned char *l1, ULong n1, ULong ofs);

/* tbase getl1(ULong i); */
/* old: void putl1(ULong i, tbase cfg, unsigned char *l1); */

static inline /* static means that getl1(i) depends from i only, opt. */
 tbase getl1(ULong i){  /* get sym-config from local index i=0..node_n1-1 */
#if  L1_PACKED  /* 2016-01-07 todo: log2 solution? but not aligned */
   tbase t1=0l; unsigned int j; /* slow but save loop, l1=(unsigned char *) */
   for(j=0;j<Bsize;j++) t1 |= ((tbase)l1[i*Bsize+j])<<(8*j);
   return t1;
#else
   tbase *t_l1 = (tbase *)l1;    return t_l1[i];
#endif
}

static inline /* static useless for void? */
void putl1(ULong i, tbase cfg, unsigned char *l1){ // Little-Endian
#if  L1_PACKED  /* 2016-01-07 todo: log2 solution? but not aligned */
  unsigned int j;
  for(j=0;j<Bsize;j++)
  l1[i*Bsize+j] = ((unsigned char)255) & (unsigned char)(cfg>>(8*j));
#else
  ((tbase *)l1)[i]= cfg; /* added 2016-01-12 speedup */
#endif
}
     
#endif
